#!/usr/bin/env python
#
# Copyright 2004      Toby Dickenson
# Copyright 2008-2011 W. Trevor King
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject
# to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""Extract a tree of module imports from a Python script.
"""

import modulefinder

class mymf(modulefinder.ModuleFinder):
    def __init__(self,*args,**kwargs):
        self._depgraph = {}
        self._types = {}
        self._paths = {}
        self._last_caller = None
        modulefinder.ModuleFinder.__init__(self,*args,**kwargs)

    def import_hook(self, name, caller=None, fromlist=None, level=-1):
        old_last_caller = self._last_caller
        try:
            self._last_caller = caller
            return modulefinder.ModuleFinder.import_hook(self,name,caller,fromlist, level)
        finally:
            self._last_caller = old_last_caller

    def import_module(self,partnam,fqname,parent):
        r = modulefinder.ModuleFinder.import_module(self,partnam,fqname,parent)
        if r is not None:
            self._depgraph.setdefault(self._last_caller.__name__,{})[r.__name__] = 1
        return r

    def load_module(self, fqname, fp, pathname, (suffix, mode, type)):
        r = modulefinder.ModuleFinder.load_module(self, fqname, fp, pathname, (suffix, mode, type))
        if r is not None:
            self._types[r.__name__] = type
            self._paths[r.__name__] = pathname
        return r


if __name__=='__main__':
    from optparse import OptionParser
    from pprint import pprint
    import sys

    usage = '%prog [options] path/to/script.py'
    p = OptionParser(usage=usage, description=__doc__)
    p.add_option('-v', '--verbose', default=0, action='count',
                 help='Increment verbosity.')

    options,args = p.parse_args()

    script = args[0]

    mf = mymf(path=sys.path[:], debug=options.verbose, excludes=[])
    mf.run_script(script)
    pprint({'depgraph':mf._depgraph,'types':mf._types,'paths':mf._paths})
